# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import re  # noqa: F401
import io
import warnings

from pydantic import validate_arguments, ValidationError
from typing_extensions import Annotated

from pydantic import Field, StrictFloat, StrictInt, StrictStr, conlist

from typing import Optional, Union

from stadiamaps.models.pelias_layer import PeliasLayer
from stadiamaps.models.pelias_response import PeliasResponse
from stadiamaps.models.pelias_source import PeliasSource

from stadiamaps.api_client import ApiClient
from stadiamaps.api_response import ApiResponse
from stadiamaps.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class GeocodingApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def autocomplete(self, text : Annotated[StrictStr, Field(..., description="The place name (address, venue name, etc.) to search for.")], focus_point_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The latitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lon`.")] = None, focus_point_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The longitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lat`.")] = None, boundary_rect_min_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the min latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_max_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the max latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_min_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the min longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_max_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the max longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_circle_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The latitude of the center of a circle to limit the search to. Requires `boundary.circle.lon`.")] = None, boundary_circle_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The longitude of the center of a circle to limit the search to. Requires `boundary.circle.lat`.")] = None, boundary_circle_radius : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.")] = None, boundary_country : Annotated[Optional[conlist(StrictStr)], Field(description="A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.")] = None, boundary_gid : Annotated[Optional[StrictStr], Field(description="The Pelias GID of an area to limit the search to.")] = None, layers : Annotated[Optional[conlist(PeliasLayer)], Field(description="A list of layers to limit the search to.")] = None, sources : Annotated[Optional[conlist(PeliasSource)], Field(description="A list of sources to limit the search to.")] = None, size : Annotated[Optional[StrictInt], Field(description="The maximum number of results to return.")] = None, lang : Annotated[Optional[StrictStr], Field(description="A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.")] = None, **kwargs) -> PeliasResponse:  # noqa: E501
        """Search and geocode quickly based on partial input.  # noqa: E501

        Autocomplete enables interactive search-as-you-type user experiences, suggesting places as you type, along with information that will help your users find the correct place quickly.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.autocomplete(text, focus_point_lat, focus_point_lon, boundary_rect_min_lat, boundary_rect_max_lat, boundary_rect_min_lon, boundary_rect_max_lon, boundary_circle_lat, boundary_circle_lon, boundary_circle_radius, boundary_country, boundary_gid, layers, sources, size, lang, async_req=True)
        >>> result = thread.get()

        :param text: The place name (address, venue name, etc.) to search for. (required)
        :type text: str
        :param focus_point_lat: The latitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lon`.
        :type focus_point_lat: float
        :param focus_point_lon: The longitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lat`.
        :type focus_point_lon: float
        :param boundary_rect_min_lat: Defines the min latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_min_lat: float
        :param boundary_rect_max_lat: Defines the max latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_max_lat: float
        :param boundary_rect_min_lon: Defines the min longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_min_lon: float
        :param boundary_rect_max_lon: Defines the max longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_max_lon: float
        :param boundary_circle_lat: The latitude of the center of a circle to limit the search to. Requires `boundary.circle.lon`.
        :type boundary_circle_lat: float
        :param boundary_circle_lon: The longitude of the center of a circle to limit the search to. Requires `boundary.circle.lat`.
        :type boundary_circle_lon: float
        :param boundary_circle_radius: The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.
        :type boundary_circle_radius: float
        :param boundary_country: A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.
        :type boundary_country: List[str]
        :param boundary_gid: The Pelias GID of an area to limit the search to.
        :type boundary_gid: str
        :param layers: A list of layers to limit the search to.
        :type layers: List[PeliasLayer]
        :param sources: A list of sources to limit the search to.
        :type sources: List[PeliasSource]
        :param size: The maximum number of results to return.
        :type size: int
        :param lang: A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.
        :type lang: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PeliasResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the autocomplete_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.autocomplete_with_http_info(text, focus_point_lat, focus_point_lon, boundary_rect_min_lat, boundary_rect_max_lat, boundary_rect_min_lon, boundary_rect_max_lon, boundary_circle_lat, boundary_circle_lon, boundary_circle_radius, boundary_country, boundary_gid, layers, sources, size, lang, **kwargs)  # noqa: E501

    @validate_arguments
    def autocomplete_with_http_info(self, text : Annotated[StrictStr, Field(..., description="The place name (address, venue name, etc.) to search for.")], focus_point_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The latitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lon`.")] = None, focus_point_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The longitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lat`.")] = None, boundary_rect_min_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the min latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_max_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the max latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_min_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the min longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_max_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the max longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_circle_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The latitude of the center of a circle to limit the search to. Requires `boundary.circle.lon`.")] = None, boundary_circle_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The longitude of the center of a circle to limit the search to. Requires `boundary.circle.lat`.")] = None, boundary_circle_radius : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.")] = None, boundary_country : Annotated[Optional[conlist(StrictStr)], Field(description="A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.")] = None, boundary_gid : Annotated[Optional[StrictStr], Field(description="The Pelias GID of an area to limit the search to.")] = None, layers : Annotated[Optional[conlist(PeliasLayer)], Field(description="A list of layers to limit the search to.")] = None, sources : Annotated[Optional[conlist(PeliasSource)], Field(description="A list of sources to limit the search to.")] = None, size : Annotated[Optional[StrictInt], Field(description="The maximum number of results to return.")] = None, lang : Annotated[Optional[StrictStr], Field(description="A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Search and geocode quickly based on partial input.  # noqa: E501

        Autocomplete enables interactive search-as-you-type user experiences, suggesting places as you type, along with information that will help your users find the correct place quickly.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.autocomplete_with_http_info(text, focus_point_lat, focus_point_lon, boundary_rect_min_lat, boundary_rect_max_lat, boundary_rect_min_lon, boundary_rect_max_lon, boundary_circle_lat, boundary_circle_lon, boundary_circle_radius, boundary_country, boundary_gid, layers, sources, size, lang, async_req=True)
        >>> result = thread.get()

        :param text: The place name (address, venue name, etc.) to search for. (required)
        :type text: str
        :param focus_point_lat: The latitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lon`.
        :type focus_point_lat: float
        :param focus_point_lon: The longitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lat`.
        :type focus_point_lon: float
        :param boundary_rect_min_lat: Defines the min latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_min_lat: float
        :param boundary_rect_max_lat: Defines the max latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_max_lat: float
        :param boundary_rect_min_lon: Defines the min longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_min_lon: float
        :param boundary_rect_max_lon: Defines the max longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_max_lon: float
        :param boundary_circle_lat: The latitude of the center of a circle to limit the search to. Requires `boundary.circle.lon`.
        :type boundary_circle_lat: float
        :param boundary_circle_lon: The longitude of the center of a circle to limit the search to. Requires `boundary.circle.lat`.
        :type boundary_circle_lon: float
        :param boundary_circle_radius: The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.
        :type boundary_circle_radius: float
        :param boundary_country: A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.
        :type boundary_country: List[str]
        :param boundary_gid: The Pelias GID of an area to limit the search to.
        :type boundary_gid: str
        :param layers: A list of layers to limit the search to.
        :type layers: List[PeliasLayer]
        :param sources: A list of sources to limit the search to.
        :type sources: List[PeliasSource]
        :param size: The maximum number of results to return.
        :type size: int
        :param lang: A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.
        :type lang: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PeliasResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'text',
            'focus_point_lat',
            'focus_point_lon',
            'boundary_rect_min_lat',
            'boundary_rect_max_lat',
            'boundary_rect_min_lon',
            'boundary_rect_max_lon',
            'boundary_circle_lat',
            'boundary_circle_lon',
            'boundary_circle_radius',
            'boundary_country',
            'boundary_gid',
            'layers',
            'sources',
            'size',
            'lang'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method autocomplete" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('text') is not None:  # noqa: E501
            _query_params.append(('text', _params['text']))

        if _params.get('focus_point_lat') is not None:  # noqa: E501
            _query_params.append(('focus.point.lat', _params['focus_point_lat']))

        if _params.get('focus_point_lon') is not None:  # noqa: E501
            _query_params.append(('focus.point.lon', _params['focus_point_lon']))

        if _params.get('boundary_rect_min_lat') is not None:  # noqa: E501
            _query_params.append(('boundary.rect.min_lat', _params['boundary_rect_min_lat']))

        if _params.get('boundary_rect_max_lat') is not None:  # noqa: E501
            _query_params.append(('boundary.rect.max_lat', _params['boundary_rect_max_lat']))

        if _params.get('boundary_rect_min_lon') is not None:  # noqa: E501
            _query_params.append(('boundary.rect.min_lon', _params['boundary_rect_min_lon']))

        if _params.get('boundary_rect_max_lon') is not None:  # noqa: E501
            _query_params.append(('boundary.rect.max_lon', _params['boundary_rect_max_lon']))

        if _params.get('boundary_circle_lat') is not None:  # noqa: E501
            _query_params.append(('boundary.circle.lat', _params['boundary_circle_lat']))

        if _params.get('boundary_circle_lon') is not None:  # noqa: E501
            _query_params.append(('boundary.circle.lon', _params['boundary_circle_lon']))

        if _params.get('boundary_circle_radius') is not None:  # noqa: E501
            _query_params.append(('boundary.circle.radius', _params['boundary_circle_radius']))

        if _params.get('boundary_country') is not None:  # noqa: E501
            _query_params.append(('boundary.country', _params['boundary_country']))
            _collection_formats['boundary.country'] = 'csv'

        if _params.get('boundary_gid') is not None:  # noqa: E501
            _query_params.append(('boundary.gid', _params['boundary_gid']))

        if _params.get('layers') is not None:  # noqa: E501
            _query_params.append(('layers', _params['layers']))
            _collection_formats['layers'] = 'csv'

        if _params.get('sources') is not None:  # noqa: E501
            _query_params.append(('sources', _params['sources']))
            _collection_formats['sources'] = 'csv'

        if _params.get('size') is not None:  # noqa: E501
            _query_params.append(('size', _params['size']))

        if _params.get('lang') is not None:  # noqa: E501
            _query_params.append(('lang', _params['lang']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['ApiKeyAuth']  # noqa: E501

        _response_types_map = {
            '200': "PeliasResponse",
            '400': None,
        }

        return self.api_client.call_api(
            '/geocoding/v1/autocomplete', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def place(self, ids : Annotated[conlist(StrictStr, min_items=1), Field(..., description="A list of Pelias GIDs to search for.")], lang : Annotated[Optional[StrictStr], Field(description="A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.")] = None, **kwargs) -> PeliasResponse:  # noqa: E501
        """Retrieve details of a place using its GID.  # noqa: E501

        Many search result components include an associated GID field (for example, an address may have a `localadmin_gid`). The place endpoint lets you look up these places directly by ID. Note that these IDs are not stable for all sources. See the [online documentation](https://docs.stadiamaps.com/geocoding-search-autocomplete/place-lookup/) for details.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.place(ids, lang, async_req=True)
        >>> result = thread.get()

        :param ids: A list of Pelias GIDs to search for. (required)
        :type ids: List[str]
        :param lang: A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.
        :type lang: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PeliasResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the place_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.place_with_http_info(ids, lang, **kwargs)  # noqa: E501

    @validate_arguments
    def place_with_http_info(self, ids : Annotated[conlist(StrictStr, min_items=1), Field(..., description="A list of Pelias GIDs to search for.")], lang : Annotated[Optional[StrictStr], Field(description="A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Retrieve details of a place using its GID.  # noqa: E501

        Many search result components include an associated GID field (for example, an address may have a `localadmin_gid`). The place endpoint lets you look up these places directly by ID. Note that these IDs are not stable for all sources. See the [online documentation](https://docs.stadiamaps.com/geocoding-search-autocomplete/place-lookup/) for details.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.place_with_http_info(ids, lang, async_req=True)
        >>> result = thread.get()

        :param ids: A list of Pelias GIDs to search for. (required)
        :type ids: List[str]
        :param lang: A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.
        :type lang: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PeliasResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'ids',
            'lang'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method place" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('ids') is not None:  # noqa: E501
            _query_params.append(('ids', _params['ids']))
            _collection_formats['ids'] = 'csv'

        if _params.get('lang') is not None:  # noqa: E501
            _query_params.append(('lang', _params['lang']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['ApiKeyAuth']  # noqa: E501

        _response_types_map = {
            '200': "PeliasResponse",
            '400': None,
        }

        return self.api_client.call_api(
            '/geocoding/v1/place', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def reverse(self, point_lat : Annotated[Union[StrictFloat, StrictInt], Field(..., description="The latitude of the point at which to perform the search.")], point_lon : Annotated[Union[StrictFloat, StrictInt], Field(..., description="The longitude of the point at which to perform the search.")], boundary_circle_radius : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.")] = None, layers : Annotated[Optional[conlist(PeliasLayer)], Field(description="A list of layers to limit the search to.")] = None, sources : Annotated[Optional[conlist(PeliasSource)], Field(description="A list of sources to limit the search to.")] = None, boundary_country : Annotated[Optional[conlist(StrictStr)], Field(description="A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.")] = None, boundary_gid : Annotated[Optional[StrictStr], Field(description="The Pelias GID of an area to limit the search to.")] = None, size : Annotated[Optional[StrictInt], Field(description="The maximum number of results to return.")] = None, lang : Annotated[Optional[StrictStr], Field(description="A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.")] = None, **kwargs) -> PeliasResponse:  # noqa: E501
        """Find places and addresses near geographic coordinates (reverse geocoding).  # noqa: E501

        Reverse geocoding and search finds places and addresses near any geographic coordinates.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.reverse(point_lat, point_lon, boundary_circle_radius, layers, sources, boundary_country, boundary_gid, size, lang, async_req=True)
        >>> result = thread.get()

        :param point_lat: The latitude of the point at which to perform the search. (required)
        :type point_lat: float
        :param point_lon: The longitude of the point at which to perform the search. (required)
        :type point_lon: float
        :param boundary_circle_radius: The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.
        :type boundary_circle_radius: float
        :param layers: A list of layers to limit the search to.
        :type layers: List[PeliasLayer]
        :param sources: A list of sources to limit the search to.
        :type sources: List[PeliasSource]
        :param boundary_country: A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.
        :type boundary_country: List[str]
        :param boundary_gid: The Pelias GID of an area to limit the search to.
        :type boundary_gid: str
        :param size: The maximum number of results to return.
        :type size: int
        :param lang: A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.
        :type lang: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PeliasResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the reverse_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.reverse_with_http_info(point_lat, point_lon, boundary_circle_radius, layers, sources, boundary_country, boundary_gid, size, lang, **kwargs)  # noqa: E501

    @validate_arguments
    def reverse_with_http_info(self, point_lat : Annotated[Union[StrictFloat, StrictInt], Field(..., description="The latitude of the point at which to perform the search.")], point_lon : Annotated[Union[StrictFloat, StrictInt], Field(..., description="The longitude of the point at which to perform the search.")], boundary_circle_radius : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.")] = None, layers : Annotated[Optional[conlist(PeliasLayer)], Field(description="A list of layers to limit the search to.")] = None, sources : Annotated[Optional[conlist(PeliasSource)], Field(description="A list of sources to limit the search to.")] = None, boundary_country : Annotated[Optional[conlist(StrictStr)], Field(description="A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.")] = None, boundary_gid : Annotated[Optional[StrictStr], Field(description="The Pelias GID of an area to limit the search to.")] = None, size : Annotated[Optional[StrictInt], Field(description="The maximum number of results to return.")] = None, lang : Annotated[Optional[StrictStr], Field(description="A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Find places and addresses near geographic coordinates (reverse geocoding).  # noqa: E501

        Reverse geocoding and search finds places and addresses near any geographic coordinates.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.reverse_with_http_info(point_lat, point_lon, boundary_circle_radius, layers, sources, boundary_country, boundary_gid, size, lang, async_req=True)
        >>> result = thread.get()

        :param point_lat: The latitude of the point at which to perform the search. (required)
        :type point_lat: float
        :param point_lon: The longitude of the point at which to perform the search. (required)
        :type point_lon: float
        :param boundary_circle_radius: The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.
        :type boundary_circle_radius: float
        :param layers: A list of layers to limit the search to.
        :type layers: List[PeliasLayer]
        :param sources: A list of sources to limit the search to.
        :type sources: List[PeliasSource]
        :param boundary_country: A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.
        :type boundary_country: List[str]
        :param boundary_gid: The Pelias GID of an area to limit the search to.
        :type boundary_gid: str
        :param size: The maximum number of results to return.
        :type size: int
        :param lang: A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.
        :type lang: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PeliasResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'point_lat',
            'point_lon',
            'boundary_circle_radius',
            'layers',
            'sources',
            'boundary_country',
            'boundary_gid',
            'size',
            'lang'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method reverse" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('point_lat') is not None:  # noqa: E501
            _query_params.append(('point.lat', _params['point_lat']))

        if _params.get('point_lon') is not None:  # noqa: E501
            _query_params.append(('point.lon', _params['point_lon']))

        if _params.get('boundary_circle_radius') is not None:  # noqa: E501
            _query_params.append(('boundary.circle.radius', _params['boundary_circle_radius']))

        if _params.get('layers') is not None:  # noqa: E501
            _query_params.append(('layers', _params['layers']))
            _collection_formats['layers'] = 'csv'

        if _params.get('sources') is not None:  # noqa: E501
            _query_params.append(('sources', _params['sources']))
            _collection_formats['sources'] = 'csv'

        if _params.get('boundary_country') is not None:  # noqa: E501
            _query_params.append(('boundary.country', _params['boundary_country']))
            _collection_formats['boundary.country'] = 'csv'

        if _params.get('boundary_gid') is not None:  # noqa: E501
            _query_params.append(('boundary.gid', _params['boundary_gid']))

        if _params.get('size') is not None:  # noqa: E501
            _query_params.append(('size', _params['size']))

        if _params.get('lang') is not None:  # noqa: E501
            _query_params.append(('lang', _params['lang']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['ApiKeyAuth']  # noqa: E501

        _response_types_map = {
            '200': "PeliasResponse",
            '400': None,
        }

        return self.api_client.call_api(
            '/geocoding/v1/reverse', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def search(self, text : Annotated[StrictStr, Field(..., description="The place name (address, venue name, etc.) to search for.")], focus_point_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The latitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lon`.")] = None, focus_point_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The longitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lat`.")] = None, boundary_rect_min_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the min latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_max_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the max latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_min_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the min longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_max_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the max longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_circle_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The latitude of the center of a circle to limit the search to. Requires `boundary.circle.lon`.")] = None, boundary_circle_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The longitude of the center of a circle to limit the search to. Requires `boundary.circle.lat`.")] = None, boundary_circle_radius : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.")] = None, boundary_country : Annotated[Optional[conlist(StrictStr)], Field(description="A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.")] = None, boundary_gid : Annotated[Optional[StrictStr], Field(description="The Pelias GID of an area to limit the search to.")] = None, layers : Annotated[Optional[conlist(PeliasLayer)], Field(description="A list of layers to limit the search to.")] = None, sources : Annotated[Optional[conlist(PeliasSource)], Field(description="A list of sources to limit the search to.")] = None, size : Annotated[Optional[StrictInt], Field(description="The maximum number of results to return.")] = None, lang : Annotated[Optional[StrictStr], Field(description="A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.")] = None, **kwargs) -> PeliasResponse:  # noqa: E501
        """Search for location and other info using a place name or address (forward geocoding).  # noqa: E501

        The search endpoint lets you search for addresses, points of interest, and administrative areas. This is most commonly used for forward geocoding applications where you need to find the geographic coordinates of an address.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.search(text, focus_point_lat, focus_point_lon, boundary_rect_min_lat, boundary_rect_max_lat, boundary_rect_min_lon, boundary_rect_max_lon, boundary_circle_lat, boundary_circle_lon, boundary_circle_radius, boundary_country, boundary_gid, layers, sources, size, lang, async_req=True)
        >>> result = thread.get()

        :param text: The place name (address, venue name, etc.) to search for. (required)
        :type text: str
        :param focus_point_lat: The latitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lon`.
        :type focus_point_lat: float
        :param focus_point_lon: The longitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lat`.
        :type focus_point_lon: float
        :param boundary_rect_min_lat: Defines the min latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_min_lat: float
        :param boundary_rect_max_lat: Defines the max latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_max_lat: float
        :param boundary_rect_min_lon: Defines the min longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_min_lon: float
        :param boundary_rect_max_lon: Defines the max longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_max_lon: float
        :param boundary_circle_lat: The latitude of the center of a circle to limit the search to. Requires `boundary.circle.lon`.
        :type boundary_circle_lat: float
        :param boundary_circle_lon: The longitude of the center of a circle to limit the search to. Requires `boundary.circle.lat`.
        :type boundary_circle_lon: float
        :param boundary_circle_radius: The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.
        :type boundary_circle_radius: float
        :param boundary_country: A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.
        :type boundary_country: List[str]
        :param boundary_gid: The Pelias GID of an area to limit the search to.
        :type boundary_gid: str
        :param layers: A list of layers to limit the search to.
        :type layers: List[PeliasLayer]
        :param sources: A list of sources to limit the search to.
        :type sources: List[PeliasSource]
        :param size: The maximum number of results to return.
        :type size: int
        :param lang: A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.
        :type lang: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PeliasResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the search_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.search_with_http_info(text, focus_point_lat, focus_point_lon, boundary_rect_min_lat, boundary_rect_max_lat, boundary_rect_min_lon, boundary_rect_max_lon, boundary_circle_lat, boundary_circle_lon, boundary_circle_radius, boundary_country, boundary_gid, layers, sources, size, lang, **kwargs)  # noqa: E501

    @validate_arguments
    def search_with_http_info(self, text : Annotated[StrictStr, Field(..., description="The place name (address, venue name, etc.) to search for.")], focus_point_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The latitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lon`.")] = None, focus_point_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The longitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lat`.")] = None, boundary_rect_min_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the min latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_max_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the max latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_min_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the min longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_max_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the max longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_circle_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The latitude of the center of a circle to limit the search to. Requires `boundary.circle.lon`.")] = None, boundary_circle_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The longitude of the center of a circle to limit the search to. Requires `boundary.circle.lat`.")] = None, boundary_circle_radius : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.")] = None, boundary_country : Annotated[Optional[conlist(StrictStr)], Field(description="A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.")] = None, boundary_gid : Annotated[Optional[StrictStr], Field(description="The Pelias GID of an area to limit the search to.")] = None, layers : Annotated[Optional[conlist(PeliasLayer)], Field(description="A list of layers to limit the search to.")] = None, sources : Annotated[Optional[conlist(PeliasSource)], Field(description="A list of sources to limit the search to.")] = None, size : Annotated[Optional[StrictInt], Field(description="The maximum number of results to return.")] = None, lang : Annotated[Optional[StrictStr], Field(description="A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Search for location and other info using a place name or address (forward geocoding).  # noqa: E501

        The search endpoint lets you search for addresses, points of interest, and administrative areas. This is most commonly used for forward geocoding applications where you need to find the geographic coordinates of an address.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.search_with_http_info(text, focus_point_lat, focus_point_lon, boundary_rect_min_lat, boundary_rect_max_lat, boundary_rect_min_lon, boundary_rect_max_lon, boundary_circle_lat, boundary_circle_lon, boundary_circle_radius, boundary_country, boundary_gid, layers, sources, size, lang, async_req=True)
        >>> result = thread.get()

        :param text: The place name (address, venue name, etc.) to search for. (required)
        :type text: str
        :param focus_point_lat: The latitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lon`.
        :type focus_point_lat: float
        :param focus_point_lon: The longitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lat`.
        :type focus_point_lon: float
        :param boundary_rect_min_lat: Defines the min latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_min_lat: float
        :param boundary_rect_max_lat: Defines the max latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_max_lat: float
        :param boundary_rect_min_lon: Defines the min longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_min_lon: float
        :param boundary_rect_max_lon: Defines the max longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_max_lon: float
        :param boundary_circle_lat: The latitude of the center of a circle to limit the search to. Requires `boundary.circle.lon`.
        :type boundary_circle_lat: float
        :param boundary_circle_lon: The longitude of the center of a circle to limit the search to. Requires `boundary.circle.lat`.
        :type boundary_circle_lon: float
        :param boundary_circle_radius: The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.
        :type boundary_circle_radius: float
        :param boundary_country: A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.
        :type boundary_country: List[str]
        :param boundary_gid: The Pelias GID of an area to limit the search to.
        :type boundary_gid: str
        :param layers: A list of layers to limit the search to.
        :type layers: List[PeliasLayer]
        :param sources: A list of sources to limit the search to.
        :type sources: List[PeliasSource]
        :param size: The maximum number of results to return.
        :type size: int
        :param lang: A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.
        :type lang: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PeliasResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'text',
            'focus_point_lat',
            'focus_point_lon',
            'boundary_rect_min_lat',
            'boundary_rect_max_lat',
            'boundary_rect_min_lon',
            'boundary_rect_max_lon',
            'boundary_circle_lat',
            'boundary_circle_lon',
            'boundary_circle_radius',
            'boundary_country',
            'boundary_gid',
            'layers',
            'sources',
            'size',
            'lang'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method search" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('text') is not None:  # noqa: E501
            _query_params.append(('text', _params['text']))

        if _params.get('focus_point_lat') is not None:  # noqa: E501
            _query_params.append(('focus.point.lat', _params['focus_point_lat']))

        if _params.get('focus_point_lon') is not None:  # noqa: E501
            _query_params.append(('focus.point.lon', _params['focus_point_lon']))

        if _params.get('boundary_rect_min_lat') is not None:  # noqa: E501
            _query_params.append(('boundary.rect.min_lat', _params['boundary_rect_min_lat']))

        if _params.get('boundary_rect_max_lat') is not None:  # noqa: E501
            _query_params.append(('boundary.rect.max_lat', _params['boundary_rect_max_lat']))

        if _params.get('boundary_rect_min_lon') is not None:  # noqa: E501
            _query_params.append(('boundary.rect.min_lon', _params['boundary_rect_min_lon']))

        if _params.get('boundary_rect_max_lon') is not None:  # noqa: E501
            _query_params.append(('boundary.rect.max_lon', _params['boundary_rect_max_lon']))

        if _params.get('boundary_circle_lat') is not None:  # noqa: E501
            _query_params.append(('boundary.circle.lat', _params['boundary_circle_lat']))

        if _params.get('boundary_circle_lon') is not None:  # noqa: E501
            _query_params.append(('boundary.circle.lon', _params['boundary_circle_lon']))

        if _params.get('boundary_circle_radius') is not None:  # noqa: E501
            _query_params.append(('boundary.circle.radius', _params['boundary_circle_radius']))

        if _params.get('boundary_country') is not None:  # noqa: E501
            _query_params.append(('boundary.country', _params['boundary_country']))
            _collection_formats['boundary.country'] = 'csv'

        if _params.get('boundary_gid') is not None:  # noqa: E501
            _query_params.append(('boundary.gid', _params['boundary_gid']))

        if _params.get('layers') is not None:  # noqa: E501
            _query_params.append(('layers', _params['layers']))
            _collection_formats['layers'] = 'csv'

        if _params.get('sources') is not None:  # noqa: E501
            _query_params.append(('sources', _params['sources']))
            _collection_formats['sources'] = 'csv'

        if _params.get('size') is not None:  # noqa: E501
            _query_params.append(('size', _params['size']))

        if _params.get('lang') is not None:  # noqa: E501
            _query_params.append(('lang', _params['lang']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['ApiKeyAuth']  # noqa: E501

        _response_types_map = {
            '200': "PeliasResponse",
            '400': None,
        }

        return self.api_client.call_api(
            '/geocoding/v1/search', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def search_structured(self, address : Annotated[Optional[StrictStr], Field(description="A street name, optionally with a house number.")] = None, neighbourhood : Annotated[Optional[StrictStr], Field(description="Varies by area, but has a locally specific meaning (NOT always an official administrative unit).")] = None, borough : Annotated[Optional[StrictStr], Field(description="A unit within a city (not widely used, but present in places like NYC and Mexico City).")] = None, locality : Annotated[Optional[StrictStr], Field(description="The city, village, town, etc. that the place/address is part of.")] = None, county : Annotated[Optional[StrictStr], Field(description="Administrative divisions between localities and regions. Not commonly used as input to structured geocoding.")] = None, region : Annotated[Optional[StrictStr], Field(description="Typically the first administrative division within a country. For example, a US state or a Canadian province.")] = None, postalcode : Annotated[Optional[StrictStr], Field(description="A mail sorting code.")] = None, country : Annotated[Optional[StrictStr], Field(description="A full name (ex: Canada), or a 2 or 3 character ISO code. Prefer ISO codes when possible.")] = None, focus_point_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The latitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lon`.")] = None, focus_point_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The longitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lat`.")] = None, boundary_rect_min_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the min latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_max_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the max latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_min_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the min longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_max_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the max longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_circle_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The latitude of the center of a circle to limit the search to. Requires `boundary.circle.lon`.")] = None, boundary_circle_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The longitude of the center of a circle to limit the search to. Requires `boundary.circle.lat`.")] = None, boundary_circle_radius : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.")] = None, boundary_country : Annotated[Optional[conlist(StrictStr)], Field(description="A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.")] = None, boundary_gid : Annotated[Optional[StrictStr], Field(description="The Pelias GID of an area to limit the search to.")] = None, layers : Annotated[Optional[conlist(PeliasLayer)], Field(description="A list of layers to limit the search to.")] = None, sources : Annotated[Optional[conlist(PeliasSource)], Field(description="A list of sources to limit the search to.")] = None, size : Annotated[Optional[StrictInt], Field(description="The maximum number of results to return.")] = None, lang : Annotated[Optional[StrictStr], Field(description="A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.")] = None, **kwargs) -> PeliasResponse:  # noqa: E501
        """Find locations matching components (structured forward geocoding).  # noqa: E501

        The structured search endpoint lets you search for addresses, points of interest, and administrative areas. Rather than a single string which the API must infer meaning from, the structured search endpoint allows you to specify the known components upfront, which is useful in many forward geocoding workflows.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.search_structured(address, neighbourhood, borough, locality, county, region, postalcode, country, focus_point_lat, focus_point_lon, boundary_rect_min_lat, boundary_rect_max_lat, boundary_rect_min_lon, boundary_rect_max_lon, boundary_circle_lat, boundary_circle_lon, boundary_circle_radius, boundary_country, boundary_gid, layers, sources, size, lang, async_req=True)
        >>> result = thread.get()

        :param address: A street name, optionally with a house number.
        :type address: str
        :param neighbourhood: Varies by area, but has a locally specific meaning (NOT always an official administrative unit).
        :type neighbourhood: str
        :param borough: A unit within a city (not widely used, but present in places like NYC and Mexico City).
        :type borough: str
        :param locality: The city, village, town, etc. that the place/address is part of.
        :type locality: str
        :param county: Administrative divisions between localities and regions. Not commonly used as input to structured geocoding.
        :type county: str
        :param region: Typically the first administrative division within a country. For example, a US state or a Canadian province.
        :type region: str
        :param postalcode: A mail sorting code.
        :type postalcode: str
        :param country: A full name (ex: Canada), or a 2 or 3 character ISO code. Prefer ISO codes when possible.
        :type country: str
        :param focus_point_lat: The latitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lon`.
        :type focus_point_lat: float
        :param focus_point_lon: The longitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lat`.
        :type focus_point_lon: float
        :param boundary_rect_min_lat: Defines the min latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_min_lat: float
        :param boundary_rect_max_lat: Defines the max latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_max_lat: float
        :param boundary_rect_min_lon: Defines the min longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_min_lon: float
        :param boundary_rect_max_lon: Defines the max longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_max_lon: float
        :param boundary_circle_lat: The latitude of the center of a circle to limit the search to. Requires `boundary.circle.lon`.
        :type boundary_circle_lat: float
        :param boundary_circle_lon: The longitude of the center of a circle to limit the search to. Requires `boundary.circle.lat`.
        :type boundary_circle_lon: float
        :param boundary_circle_radius: The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.
        :type boundary_circle_radius: float
        :param boundary_country: A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.
        :type boundary_country: List[str]
        :param boundary_gid: The Pelias GID of an area to limit the search to.
        :type boundary_gid: str
        :param layers: A list of layers to limit the search to.
        :type layers: List[PeliasLayer]
        :param sources: A list of sources to limit the search to.
        :type sources: List[PeliasSource]
        :param size: The maximum number of results to return.
        :type size: int
        :param lang: A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.
        :type lang: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PeliasResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the search_structured_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.search_structured_with_http_info(address, neighbourhood, borough, locality, county, region, postalcode, country, focus_point_lat, focus_point_lon, boundary_rect_min_lat, boundary_rect_max_lat, boundary_rect_min_lon, boundary_rect_max_lon, boundary_circle_lat, boundary_circle_lon, boundary_circle_radius, boundary_country, boundary_gid, layers, sources, size, lang, **kwargs)  # noqa: E501

    @validate_arguments
    def search_structured_with_http_info(self, address : Annotated[Optional[StrictStr], Field(description="A street name, optionally with a house number.")] = None, neighbourhood : Annotated[Optional[StrictStr], Field(description="Varies by area, but has a locally specific meaning (NOT always an official administrative unit).")] = None, borough : Annotated[Optional[StrictStr], Field(description="A unit within a city (not widely used, but present in places like NYC and Mexico City).")] = None, locality : Annotated[Optional[StrictStr], Field(description="The city, village, town, etc. that the place/address is part of.")] = None, county : Annotated[Optional[StrictStr], Field(description="Administrative divisions between localities and regions. Not commonly used as input to structured geocoding.")] = None, region : Annotated[Optional[StrictStr], Field(description="Typically the first administrative division within a country. For example, a US state or a Canadian province.")] = None, postalcode : Annotated[Optional[StrictStr], Field(description="A mail sorting code.")] = None, country : Annotated[Optional[StrictStr], Field(description="A full name (ex: Canada), or a 2 or 3 character ISO code. Prefer ISO codes when possible.")] = None, focus_point_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The latitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lon`.")] = None, focus_point_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The longitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lat`.")] = None, boundary_rect_min_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the min latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_max_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the max latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_min_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the min longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_rect_max_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="Defines the max longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.")] = None, boundary_circle_lat : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The latitude of the center of a circle to limit the search to. Requires `boundary.circle.lon`.")] = None, boundary_circle_lon : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The longitude of the center of a circle to limit the search to. Requires `boundary.circle.lat`.")] = None, boundary_circle_radius : Annotated[Optional[Union[StrictFloat, StrictInt]], Field(description="The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.")] = None, boundary_country : Annotated[Optional[conlist(StrictStr)], Field(description="A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.")] = None, boundary_gid : Annotated[Optional[StrictStr], Field(description="The Pelias GID of an area to limit the search to.")] = None, layers : Annotated[Optional[conlist(PeliasLayer)], Field(description="A list of layers to limit the search to.")] = None, sources : Annotated[Optional[conlist(PeliasSource)], Field(description="A list of sources to limit the search to.")] = None, size : Annotated[Optional[StrictInt], Field(description="The maximum number of results to return.")] = None, lang : Annotated[Optional[StrictStr], Field(description="A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Find locations matching components (structured forward geocoding).  # noqa: E501

        The structured search endpoint lets you search for addresses, points of interest, and administrative areas. Rather than a single string which the API must infer meaning from, the structured search endpoint allows you to specify the known components upfront, which is useful in many forward geocoding workflows.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.search_structured_with_http_info(address, neighbourhood, borough, locality, county, region, postalcode, country, focus_point_lat, focus_point_lon, boundary_rect_min_lat, boundary_rect_max_lat, boundary_rect_min_lon, boundary_rect_max_lon, boundary_circle_lat, boundary_circle_lon, boundary_circle_radius, boundary_country, boundary_gid, layers, sources, size, lang, async_req=True)
        >>> result = thread.get()

        :param address: A street name, optionally with a house number.
        :type address: str
        :param neighbourhood: Varies by area, but has a locally specific meaning (NOT always an official administrative unit).
        :type neighbourhood: str
        :param borough: A unit within a city (not widely used, but present in places like NYC and Mexico City).
        :type borough: str
        :param locality: The city, village, town, etc. that the place/address is part of.
        :type locality: str
        :param county: Administrative divisions between localities and regions. Not commonly used as input to structured geocoding.
        :type county: str
        :param region: Typically the first administrative division within a country. For example, a US state or a Canadian province.
        :type region: str
        :param postalcode: A mail sorting code.
        :type postalcode: str
        :param country: A full name (ex: Canada), or a 2 or 3 character ISO code. Prefer ISO codes when possible.
        :type country: str
        :param focus_point_lat: The latitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lon`.
        :type focus_point_lat: float
        :param focus_point_lon: The longitude of the point to focus the search on. This will bias results toward the focus point. Requires `focus.point.lat`.
        :type focus_point_lon: float
        :param boundary_rect_min_lat: Defines the min latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_min_lat: float
        :param boundary_rect_max_lat: Defines the max latitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_max_lat: float
        :param boundary_rect_min_lon: Defines the min longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_min_lon: float
        :param boundary_rect_max_lon: Defines the max longitude component of a bounding box to limit the search to. Requires all other `boundary.rect` parameters to be specified.
        :type boundary_rect_max_lon: float
        :param boundary_circle_lat: The latitude of the center of a circle to limit the search to. Requires `boundary.circle.lon`.
        :type boundary_circle_lat: float
        :param boundary_circle_lon: The longitude of the center of a circle to limit the search to. Requires `boundary.circle.lat`.
        :type boundary_circle_lon: float
        :param boundary_circle_radius: The radius of the circle (in kilometers) to limit the search to. Defaults to 50km if unspecified.
        :type boundary_circle_radius: float
        :param boundary_country: A list of countries to limit the search to. These may be either full names (ex: Canada), or an ISO 3116-1 alpha-2 or alpha-3 code. Prefer ISO codes when possible.
        :type boundary_country: List[str]
        :param boundary_gid: The Pelias GID of an area to limit the search to.
        :type boundary_gid: str
        :param layers: A list of layers to limit the search to.
        :type layers: List[PeliasLayer]
        :param sources: A list of sources to limit the search to.
        :type sources: List[PeliasSource]
        :param size: The maximum number of results to return.
        :type size: int
        :param lang: A BCP47 language tag which specifies a preference for localization of results. By default, results are in the default locale of the source data, but specifying a language will attempt to localize the results. Note that while a `langtag` (in RFC 5646 terms) can contain script, region, etc., only the `language` portion, an ISO 639 code, will be considered. So `en-US` and `en-GB` will both be treated as English.
        :type lang: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PeliasResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'address',
            'neighbourhood',
            'borough',
            'locality',
            'county',
            'region',
            'postalcode',
            'country',
            'focus_point_lat',
            'focus_point_lon',
            'boundary_rect_min_lat',
            'boundary_rect_max_lat',
            'boundary_rect_min_lon',
            'boundary_rect_max_lon',
            'boundary_circle_lat',
            'boundary_circle_lon',
            'boundary_circle_radius',
            'boundary_country',
            'boundary_gid',
            'layers',
            'sources',
            'size',
            'lang'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method search_structured" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('address') is not None:  # noqa: E501
            _query_params.append(('address', _params['address']))

        if _params.get('neighbourhood') is not None:  # noqa: E501
            _query_params.append(('neighbourhood', _params['neighbourhood']))

        if _params.get('borough') is not None:  # noqa: E501
            _query_params.append(('borough', _params['borough']))

        if _params.get('locality') is not None:  # noqa: E501
            _query_params.append(('locality', _params['locality']))

        if _params.get('county') is not None:  # noqa: E501
            _query_params.append(('county', _params['county']))

        if _params.get('region') is not None:  # noqa: E501
            _query_params.append(('region', _params['region']))

        if _params.get('postalcode') is not None:  # noqa: E501
            _query_params.append(('postalcode', _params['postalcode']))

        if _params.get('country') is not None:  # noqa: E501
            _query_params.append(('country', _params['country']))

        if _params.get('focus_point_lat') is not None:  # noqa: E501
            _query_params.append(('focus.point.lat', _params['focus_point_lat']))

        if _params.get('focus_point_lon') is not None:  # noqa: E501
            _query_params.append(('focus.point.lon', _params['focus_point_lon']))

        if _params.get('boundary_rect_min_lat') is not None:  # noqa: E501
            _query_params.append(('boundary.rect.min_lat', _params['boundary_rect_min_lat']))

        if _params.get('boundary_rect_max_lat') is not None:  # noqa: E501
            _query_params.append(('boundary.rect.max_lat', _params['boundary_rect_max_lat']))

        if _params.get('boundary_rect_min_lon') is not None:  # noqa: E501
            _query_params.append(('boundary.rect.min_lon', _params['boundary_rect_min_lon']))

        if _params.get('boundary_rect_max_lon') is not None:  # noqa: E501
            _query_params.append(('boundary.rect.max_lon', _params['boundary_rect_max_lon']))

        if _params.get('boundary_circle_lat') is not None:  # noqa: E501
            _query_params.append(('boundary.circle.lat', _params['boundary_circle_lat']))

        if _params.get('boundary_circle_lon') is not None:  # noqa: E501
            _query_params.append(('boundary.circle.lon', _params['boundary_circle_lon']))

        if _params.get('boundary_circle_radius') is not None:  # noqa: E501
            _query_params.append(('boundary.circle.radius', _params['boundary_circle_radius']))

        if _params.get('boundary_country') is not None:  # noqa: E501
            _query_params.append(('boundary.country', _params['boundary_country']))
            _collection_formats['boundary.country'] = 'csv'

        if _params.get('boundary_gid') is not None:  # noqa: E501
            _query_params.append(('boundary.gid', _params['boundary_gid']))

        if _params.get('layers') is not None:  # noqa: E501
            _query_params.append(('layers', _params['layers']))
            _collection_formats['layers'] = 'csv'

        if _params.get('sources') is not None:  # noqa: E501
            _query_params.append(('sources', _params['sources']))
            _collection_formats['sources'] = 'csv'

        if _params.get('size') is not None:  # noqa: E501
            _query_params.append(('size', _params['size']))

        if _params.get('lang') is not None:  # noqa: E501
            _query_params.append(('lang', _params['lang']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['ApiKeyAuth']  # noqa: E501

        _response_types_map = {
            '200': "PeliasResponse",
            '400': None,
        }

        return self.api_client.call_api(
            '/geocoding/v1/search/structured', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictBool, StrictInt
from stadiamaps.models.map_match_trace_options import MapMatchTraceOptions

class MapMatchRequestAllOf(BaseModel):
    """
    MapMatchRequestAllOf
    """
    begin_time: Optional[StrictInt] = Field(None, description="The timestamp at the start of the trace. Combined with `durations`, this provides a way to include timing information for an `encoded_polyline` trace.")
    durations: Optional[StrictInt] = Field(None, description="A list of durations (in seconds) between each successive pair of points in a polyline.")
    use_timestamps: Optional[StrictBool] = Field(False, description="If true, the input timestamps or durations should be used when computing elapsed time for each edge along the matched path rather than the routing algorithm estimates.")
    trace_options: Optional[MapMatchTraceOptions] = None
    linear_references: Optional[StrictBool] = Field(False, description="If true, the response will include a `linear_references` value that contains an array of base64-encoded [OpenLR location references](https://www.openlr-association.com/fileadmin/user_upload/openlr-whitepaper_v1.5.pdf), one for each graph edge of the road network matched by the trace.")
    additional_properties: Dict[str, Any] = {}
    __properties = ["begin_time", "durations", "use_timestamps", "trace_options", "linear_references"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> MapMatchRequestAllOf:
        """Create an instance of MapMatchRequestAllOf from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of trace_options
        if self.trace_options:
            _dict['trace_options'] = self.trace_options.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> MapMatchRequestAllOf:
        """Create an instance of MapMatchRequestAllOf from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return MapMatchRequestAllOf.parse_obj(obj)

        _obj = MapMatchRequestAllOf.parse_obj({
            "begin_time": obj.get("begin_time"),
            "durations": obj.get("durations"),
            "use_timestamps": obj.get("use_timestamps") if obj.get("use_timestamps") is not None else False,
            "trace_options": MapMatchTraceOptions.from_dict(obj.get("trace_options")) if obj.get("trace_options") is not None else None,
            "linear_references": obj.get("linear_references") if obj.get("linear_references") is not None else False
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class CostingModel(str, Enum):
    """
    Costing models for determining the most optimal route to take. Note that bikeshare and motorcycle are still in beta. While Valhalla supports multimodal routing, we do not currently process transit data and have excluded it from the list. See https://valhalla.readthedocs.io/en/latest/api/turn-by-turn/api-reference/#costing-models for detailed descriptions of each model.
    """

    """
    allowed enum values
    """
    AUTO = 'auto'
    BUS = 'bus'
    TAXI = 'taxi'
    TRUCK = 'truck'
    BICYCLE = 'bicycle'
    BIKESHARE = 'bikeshare'
    MOTOR_SCOOTER = 'motor_scooter'
    MOTORCYCLE = 'motorcycle'
    PEDESTRIAN = 'pedestrian'

    @classmethod
    def from_json(cls, json_str: str) -> CostingModel:
        """Create an instance of CostingModel from a JSON string"""
        return CostingModel(json.loads(json_str))





# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional, Union
from pydantic import BaseModel, Field, StrictFloat, StrictInt, StrictStr, conlist
from stadiamaps.models.costing_model import CostingModel
from stadiamaps.models.costing_options import CostingOptions
from stadiamaps.models.directions_options import DirectionsOptions
from stadiamaps.models.routing_waypoint import RoutingWaypoint

class RouteRequest(BaseModel):
    """
    RouteRequest
    """
    id: Optional[StrictStr] = Field(None, description="An identifier to disambiguate requests (echoed by the server).")
    locations: conlist(RoutingWaypoint, min_items=2) = Field(...)
    costing: CostingModel = Field(...)
    costing_options: Optional[CostingOptions] = None
    avoid_locations: Optional[conlist(RoutingWaypoint)] = None
    avoid_polygons: Optional[conlist(conlist(conlist(Union[StrictFloat, StrictInt])))] = Field(None, description="One or multiple exterior rings of polygons in the form of nested JSON arrays. Roads intersecting these rings will be avoided during path finding. Open rings will be closed automatically.")
    directions_options: Optional[DirectionsOptions] = None
    additional_properties: Dict[str, Any] = {}
    __properties = ["id", "locations", "costing", "costing_options", "avoid_locations", "avoid_polygons", "directions_options"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RouteRequest:
        """Create an instance of RouteRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in locations (list)
        _items = []
        if self.locations:
            for _item in self.locations:
                if _item:
                    _items.append(_item.to_dict())
            _dict['locations'] = _items
        # override the default output from pydantic by calling `to_dict()` of costing_options
        if self.costing_options:
            _dict['costing_options'] = self.costing_options.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in avoid_locations (list)
        _items = []
        if self.avoid_locations:
            for _item in self.avoid_locations:
                if _item:
                    _items.append(_item.to_dict())
            _dict['avoid_locations'] = _items
        # override the default output from pydantic by calling `to_dict()` of directions_options
        if self.directions_options:
            _dict['directions_options'] = self.directions_options.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RouteRequest:
        """Create an instance of RouteRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RouteRequest.parse_obj(obj)

        _obj = RouteRequest.parse_obj({
            "id": obj.get("id"),
            "locations": [RoutingWaypoint.from_dict(_item) for _item in obj.get("locations")] if obj.get("locations") is not None else None,
            "costing": obj.get("costing"),
            "costing_options": CostingOptions.from_dict(obj.get("costing_options")) if obj.get("costing_options") is not None else None,
            "avoid_locations": [RoutingWaypoint.from_dict(_item) for _item in obj.get("avoid_locations")] if obj.get("avoid_locations") is not None else None,
            "avoid_polygons": obj.get("avoid_polygons"),
            "directions_options": DirectionsOptions.from_dict(obj.get("directions_options")) if obj.get("directions_options") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




import os
import unittest

import stadiamaps

location_a = {"lat": 40.042072, "lon": -76.306572}
location_b = {"lat": 39.992115, "lon": -76.781559}
location_c = {"lat": 39.984519, "lon": -76.6956}


class TestRouting(unittest.TestCase):
    def setUp(self):
        self.configuration = stadiamaps.Configuration()
        self.configuration.api_key["ApiKeyAuth"] = os.environ["STADIA_API_KEY"]

    def tearDown(self):
        pass

    def testRoute(self):
        with stadiamaps.ApiClient(self.configuration) as api_client:
            api_instance = stadiamaps.RoutingApi(api_client)

            req = stadiamaps.RouteRequest(
                id="route",
                locations=[
                    stadiamaps.RoutingWaypoint.from_dict(location_a),
                    stadiamaps.RoutingWaypoint.from_dict(location_b)
                ],
                costing=stadiamaps.CostingModel.AUTO,
                costing_options=stadiamaps.CostingOptions(auto=stadiamaps.AutoCostingOptions(use_tolls=1)),
                directions_options=stadiamaps.DirectionsOptions(units=stadiamaps.DistanceUnit.MI)
            )

            res = api_instance.route(req)
            self.assertEqual(req.id, res.id)
            self.assertEqual(0, res.trip.status)
            self.assertEqual("miles", res.trip.units)
            self.assertEqual(len(res.trip.legs), 1)

    def testOptimizedRoute(self):
        with stadiamaps.ApiClient(self.configuration) as api_client:
            api_instance = stadiamaps.RoutingApi(api_client)

            req = stadiamaps.OptimizedRouteRequest(
                id="optimized_route",
                locations=[
                    stadiamaps.Coordinate.from_dict(location_a),
                    stadiamaps.Coordinate.from_dict(location_b),
                    stadiamaps.Coordinate.from_dict(location_c),
                    stadiamaps.Coordinate.from_dict(location_a),
                ],
                costing=stadiamaps.CostingModel.AUTO,
                costing_options=stadiamaps.CostingOptions(auto=stadiamaps.AutoCostingOptions(use_tolls=0.7)),
                directions_options=stadiamaps.DirectionsOptions(units=stadiamaps.DistanceUnit.MI)
            )

            res = api_instance.optimized_route(req)
            self.assertEqual(req.id, res.id)
            self.assertEqual(0, res.trip.status)
            self.assertEqual("miles", res.trip.units)
            self.assertGreater(len(res.trip.legs), 1)

    def testTimeDistanceMatrix(self):
        with stadiamaps.ApiClient(self.configuration) as api_client:
            api_instance = stadiamaps.RoutingApi(api_client)

            req = stadiamaps.MatrixRequest(
                id="matrix",
                sources=[
                    stadiamaps.Coordinate.from_dict(location_a),
                ],
                targets=[
                    stadiamaps.Coordinate.from_dict(location_b),
                    stadiamaps.Coordinate.from_dict(location_c),
                ],
                costing=stadiamaps.MatrixCostingModel.PEDESTRIAN,
            )

            res = api_instance.time_distance_matrix(req)
            self.assertEqual(req.id, res.id)
            self.assertEqual([req.sources], res.sources)
            self.assertEqual([req.targets], res.targets)
            self.assertGreaterEqual(len(res.sources_to_targets), 1)
            self.assertEqual("kilometers", res.units)

    def testNearestRoads(self):
        with stadiamaps.ApiClient(self.configuration) as api_client:
            api_instance = stadiamaps.RoutingApi(api_client)

            req = stadiamaps.NearestRoadsRequest(
                locations=[location_a]
            )
            res = api_instance.nearest_roads(req)
            self.assertGreaterEqual(len(res), 1)
            self.assertGreaterEqual(len(res[0].edges), 1)

    def testIsochrone(self):
        with stadiamaps.ApiClient(self.configuration) as api_client:
            api_instance = stadiamaps.RoutingApi(api_client)

            req = stadiamaps.IsochroneRequest(
                id="isochrone",
                locations=[location_a],
                costing=stadiamaps.IsochroneCostingModel.PEDESTRIAN,
                contours=[
                    stadiamaps.Contour(time=5, color="aabbcc")
                ],
                polygons=True,
            )
            res = api_instance.isochrone(req)
            self.assertEqual(req.id, res.id)
            self.assertGreaterEqual(len(res.features), 1)

    def testMapMatch(self):
        with stadiamaps.ApiClient(self.configuration) as api_client:
            api_instance = stadiamaps.RoutingApi(api_client)

            req = stadiamaps.MapMatchRequest(
                id="map_match",
                encoded_polyline="_grbgAh~{nhF?lBAzBFvBHxBEtBKdB?fB@dBZdBb@hBh@jBb@x@\\|@x@pB\\x@v@hBl@nBPbCXtBn@|@z@ZbAEbAa@~@q@z@QhA]pAUpAVhAPlAWtASpAAdA[dASdAQhAIlARjANnAZhAf@n@`A?lB^nCRbA\\xB`@vBf@tBTbCFbARzBZvBThBRnBNrBP`CHbCF`CNdCb@vBX`ARlAJfADhA@dAFdAP`AR`Ah@hBd@bBl@rBV|B?vB]tBCvBBhAF`CFnBXtAVxAVpAVtAb@|AZ`Bd@~BJfA@fAHdADhADhABjAGzAInAAjAB|BNbCR|BTjBZtB`@lBh@lB\\|Bl@rBXtBN`Al@g@t@?nAA~AKvACvAAlAMdAU`Ac@hAShAI`AJ`AIdAi@bAu@|@k@p@]p@a@bAc@z@g@~@Ot@Bz@f@X`BFtBXdCLbAf@zBh@fBb@xAb@nATjAKjAW`BI|AEpAHjAPdAAfAGdAFjAv@p@XlAVnA?~A?jAInAPtAVxAXnAf@tBDpBJpBXhBJfBDpAZ|Ax@pAz@h@~@lA|@bAnAd@hAj@tAR~AKxAc@xAShA]hAIdAAjA]~A[v@BhB?dBSv@Ct@CvAI~@Oz@Pv@dAz@lAj@~A^`B^|AXvAVpAXdBh@~Ap@fCh@hB\\zBN`Aj@xBFdA@jALbAPbAJdAHdAJbAHbAHfAJhALbA\\lBTvBAdC@bC@jCKjASbC?`CM`CDpB\\xAj@tB\\fA\\bAVfAJdAJbAXz@L|BO`AOdCDdA@~B\\z@l@v@l@v@l@r@j@t@b@x@b@r@z@jBVfCJdAJdANbCPfCF|BRhBS~BS`AYbAe@~BQdA",
                costing=stadiamaps.MapMatchCostingModel.PEDESTRIAN,
                directions_options=stadiamaps.DirectionsOptions(units=stadiamaps.DistanceUnit.MI),
                linear_references=True,
            )
            res = api_instance.map_match(req)
            self.assertEqual(req.id, res.id)
            self.assertEqual(0, res.trip.status)
            self.assertEqual("miles", res.trip.units)
            self.assertEqual(len(res.trip.legs), 1)

    def testTraceAttributes(self):
        with stadiamaps.ApiClient(self.configuration) as api_client:
            api_instance = stadiamaps.RoutingApi(api_client)

            req = stadiamaps.TraceAttributesRequest(
                id="trace",
                encoded_polyline="_grbgAh~{nhF?lBAzBFvBHxBEtBKdB?fB@dBZdBb@hBh@jBb@x@\\|@x@pB\\x@v@hBl@nBPbCXtBn@|@z@ZbAEbAa@~@q@z@QhA]pAUpAVhAPlAWtASpAAdA[dASdAQhAIlARjANnAZhAf@n@`A?lB^nCRbA\\xB`@vBf@tBTbCFbARzBZvBThBRnBNrBP`CHbCF`CNdCb@vBX`ARlAJfADhA@dAFdAP`AR`Ah@hBd@bBl@rBV|B?vB]tBCvBBhAF`CFnBXtAVxAVpAVtAb@|AZ`Bd@~BJfA@fAHdADhADhABjAGzAInAAjAB|BNbCR|BTjBZtB`@lBh@lB\\|Bl@rBXtBN`Al@g@t@?nAA~AKvACvAAlAMdAU`Ac@hAShAI`AJ`AIdAi@bAu@|@k@p@]p@a@bAc@z@g@~@Ot@Bz@f@X`BFtBXdCLbAf@zBh@fBb@xAb@nATjAKjAW`BI|AEpAHjAPdAAfAGdAFjAv@p@XlAVnA?~A?jAInAPtAVxAXnAf@tBDpBJpBXhBJfBDpAZ|Ax@pAz@h@~@lA|@bAnAd@hAj@tAR~AKxAc@xAShA]hAIdAAjA]~A[v@BhB?dBSv@Ct@CvAI~@Oz@Pv@dAz@lAj@~A^`B^|AXvAVpAXdBh@~Ap@fCh@hB\\zBN`Aj@xBFdA@jALbAPbAJdAHdAJbAHbAHfAJhALbA\\lBTvBAdC@bC@jCKjASbC?`CM`CDpB\\xAj@tB\\fA\\bAVfAJdAJbAXz@L|BO`AOdCDdA@~B\\z@l@v@l@v@l@r@j@t@b@x@b@r@z@jBVfCJdAJdANbCPfCF|BRhBS~BS`AYbAe@~BQdA",
                costing=stadiamaps.MapMatchCostingModel.PEDESTRIAN,
                directions_options=stadiamaps.DirectionsOptions(units=stadiamaps.DistanceUnit.MI),
            )
            res = api_instance.trace_attributes(req)
            self.assertEqual(req.id, res.id)
            self.assertEqual("miles", res.units)
            self.assertGreaterEqual(len(res.admins), 1)
            self.assertGreater(len(res.edges), 1)
            self.assertGreater(len(res.matched_points), 1)
            self.assertGreater(len(res.shape), 1)



# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class PeliasLayer(str, Enum):
    """
    Our database is organized into several layers (in the GIS sense of the term) as follows:  - `venue`: Points of interest, businesses, and things with walls - `address`: Places with a street address - `street`: Streets, roads, highways - `county`: Places that issue passports, nations, nation-states - `macroregion`: A related group of regions (mostly in Europe) - `region`: The first administrative division within a country (usually states and provinces) - `macrocounty`: A related group of counties (mostly in Europe) - `county`: Official governmental areas; usually bigger than a locality, but almost always smaller than a region - `locality`: Towns, hamlets, cities, etc. - `localadmin`: Local administrative boundaries - `borough`: Local administrative boundaries within cities (not widely used, but present in places such as NYC and Mexico City) - `neighbourhood`: Social communities and neighborhoods (note the British spelling in the API!) - `postalcode`: Postal codes used by mail services (note: not used for reverse geocoding) - `coarse`: An alias for simultaneously using all administrative layers (everything except `venue` and `address`) - `marinearea`: Places with fishes and boats. - `ocean`: A really big marine area. 
    """

    """
    allowed enum values
    """
    VENUE = 'venue'
    ADDRESS = 'address'
    STREET = 'street'
    COUNTRY = 'country'
    MACROREGION = 'macroregion'
    REGION = 'region'
    MACROCOUNTY = 'macrocounty'
    COUNTY = 'county'
    LOCALITY = 'locality'
    LOCALADMIN = 'localadmin'
    BOROUGH = 'borough'
    NEIGHBOURHOOD = 'neighbourhood'
    POSTALCODE = 'postalcode'
    COARSE = 'coarse'
    DEPENDENCY = 'dependency'
    MACROHOOD = 'macrohood'
    MARINEAREA = 'marinearea'
    DISPUTED = 'disputed'
    EMPIRE = 'empire'
    CONTINENT = 'continent'
    OCEAN = 'ocean'

    @classmethod
    def from_json(cls, json_str: str) -> PeliasLayer:
        """Create an instance of PeliasLayer from a JSON string"""
        return PeliasLayer(json.loads(json_str))





import os
import unittest

import stadiamaps

address = "Põhja pst 27a"


class TestGeocoding(unittest.TestCase):
    def setUp(self):
        self.configuration = stadiamaps.Configuration()
        self.configuration.api_key["ApiKeyAuth"] = os.environ["STADIA_API_KEY"]

    def tearDown(self):
        pass

    def testAutocomplete(self):
        with stadiamaps.ApiClient(self.configuration) as api_client:
            api_instance = stadiamaps.GeocodingApi(api_client)

            res = api_instance.autocomplete(text=address)
            self.assertEqual("Estonia", res.features[0].properties.country)
            self.assertEqual("address", res.features[0].properties.layer)

    def testSearch(self):
        with stadiamaps.ApiClient(self.configuration) as api_client:
            api_instance = stadiamaps.GeocodingApi(api_client)

            res = api_instance.search(address)
            self.assertEqual("Estonia", res.features[0].properties.country)
            self.assertEqual("address", res.features[0].properties.layer)

    def testStructuredSearch(self):
        with stadiamaps.ApiClient(self.configuration) as api_client:
            api_instance = stadiamaps.GeocodingApi(api_client)

            res = api_instance.search_structured(address=address, country="Estonia")
            self.assertEqual("Estonia", res.features[0].properties.country)
            self.assertEqual("address", res.features[0].properties.layer)

    def testReverse(self):
        with stadiamaps.ApiClient(self.configuration) as api_client:
            api_instance = stadiamaps.GeocodingApi(api_client)

            res = api_instance.reverse(59.444351, 24.750645)
            self.assertEqual("Estonia", res.features[0].properties.country)
            self.assertEqual("address", res.features[0].properties.layer)

    def testReverseUncommonLayer(self):
        with stadiamaps.ApiClient(self.configuration) as api_client:
            api_instance = stadiamaps.GeocodingApi(api_client)

            # Middle of the Gulf of Oman
            res = api_instance.reverse(24.750645, 59.444351)
            self.assertEqual("marinearea", res.features[0].properties.layer)

    def testPlace(self):
        with stadiamaps.ApiClient(self.configuration) as api_client:
            api_instance = stadiamaps.GeocodingApi(api_client)

            res = api_instance.place(["openstreetmap:address:way/109867749"])
            self.assertEqual(1, len(res.features))
            self.assertEqual("Estonia", res.features[0].properties.country)
            self.assertEqual("address", res.features[0].properties.layer)



# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, StrictStr, conlist

class EdgeSign(BaseModel):
    """
    EdgeSign
    """
    exit_number: Optional[conlist(StrictStr)] = None
    exit_branch: Optional[conlist(StrictStr)] = None
    exit_toward: Optional[conlist(StrictStr)] = None
    exit_name: Optional[conlist(StrictStr)] = None
    additional_properties: Dict[str, Any] = {}
    __properties = ["exit_number", "exit_branch", "exit_toward", "exit_name"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> EdgeSign:
        """Create an instance of EdgeSign from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> EdgeSign:
        """Create an instance of EdgeSign from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return EdgeSign.parse_obj(obj)

        _obj = EdgeSign.parse_obj({
            "exit_number": obj.get("exit_number"),
            "exit_branch": obj.get("exit_branch"),
            "exit_toward": obj.get("exit_toward"),
            "exit_name": obj.get("exit_name")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt

class TruckCostingOptionsAllOf(BaseModel):
    """
    TruckCostingOptionsAllOf
    """
    height: Optional[Union[StrictFloat, StrictInt]] = Field(4.11, description="The height of the truck (in meters).")
    width: Optional[Union[StrictFloat, StrictInt]] = Field(2.6, description="The width of the truck (in meters).")
    length: Optional[Union[StrictFloat, StrictInt]] = Field(21.64, description="The length of the truck (in meters).")
    weight: Optional[Union[StrictFloat, StrictInt]] = Field(21.77, description="The weight of the truck (in tonnes).")
    axle_load: Optional[Union[StrictFloat, StrictInt]] = Field(9.07, description="The axle load of the truck (in tonnes).")
    hazmat: Optional[StrictBool] = Field(False, description="Whether or not the truck is carrying hazardous materials.")
    additional_properties: Dict[str, Any] = {}
    __properties = ["height", "width", "length", "weight", "axle_load", "hazmat"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TruckCostingOptionsAllOf:
        """Create an instance of TruckCostingOptionsAllOf from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TruckCostingOptionsAllOf:
        """Create an instance of TruckCostingOptionsAllOf from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TruckCostingOptionsAllOf.parse_obj(obj)

        _obj = TruckCostingOptionsAllOf.parse_obj({
            "height": obj.get("height") if obj.get("height") is not None else 4.11,
            "width": obj.get("width") if obj.get("width") is not None else 2.6,
            "length": obj.get("length") if obj.get("length") is not None else 21.64,
            "weight": obj.get("weight") if obj.get("weight") is not None else 21.77,
            "axle_load": obj.get("axle_load") if obj.get("axle_load") is not None else 9.07,
            "hazmat": obj.get("hazmat") if obj.get("hazmat") is not None else False
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist, validator
from stadiamaps.models.costing_options import CostingOptions
from stadiamaps.models.directions_options import DirectionsOptions
from stadiamaps.models.map_match_costing_model import MapMatchCostingModel
from stadiamaps.models.map_match_waypoint import MapMatchWaypoint
from stadiamaps.models.trace_attributes_request_all_of_filters import TraceAttributesRequestAllOfFilters

class TraceAttributesRequest(BaseModel):
    """
    TraceAttributesRequest
    """
    id: Optional[StrictStr] = Field(None, description="An identifier to disambiguate requests (echoed by the server).")
    shape: Optional[conlist(MapMatchWaypoint)] = Field(None, description="REQUIRED if `encoded_polyline` is not present. Note that `break` type locations are only supported when `shape_match` is set to `map_match`.")
    encoded_polyline: Optional[StrictStr] = Field(None, description="REQUIRED if `shape` is not present. An encoded polyline (https://developers.google.com/maps/documentation/utilities/polylinealgorithm). Note that the polyline must be encoded with 6 digits of precision rather than the usual 5.")
    costing: MapMatchCostingModel = Field(...)
    costing_options: Optional[CostingOptions] = None
    shape_match: Optional[StrictStr] = Field(None, description="Three snapping modes provide some control over how the map matching occurs. `edge_walk` is fast, but requires extremely precise data that matches the route graph almost perfectly. `map_snap` can handle significantly noisier data, but is very expensive. `walk_or_snap`, the default, tries to use edge walking first and falls back to map matching if edge walking fails. In general, you should not need to change this parameter unless you want to trace a multi-leg route with multiple `break` locations in the `shape`.")
    directions_options: Optional[DirectionsOptions] = None
    filters: Optional[TraceAttributesRequestAllOfFilters] = None
    additional_properties: Dict[str, Any] = {}
    __properties = ["id", "shape", "encoded_polyline", "costing", "costing_options", "shape_match", "directions_options", "filters"]

    @validator('shape_match')
    def shape_match_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('edge_walk', 'map_snap', 'walk_or_snap'):
            raise ValueError("must be one of enum values ('edge_walk', 'map_snap', 'walk_or_snap')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TraceAttributesRequest:
        """Create an instance of TraceAttributesRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in shape (list)
        _items = []
        if self.shape:
            for _item in self.shape:
                if _item:
                    _items.append(_item.to_dict())
            _dict['shape'] = _items
        # override the default output from pydantic by calling `to_dict()` of costing_options
        if self.costing_options:
            _dict['costing_options'] = self.costing_options.to_dict()
        # override the default output from pydantic by calling `to_dict()` of directions_options
        if self.directions_options:
            _dict['directions_options'] = self.directions_options.to_dict()
        # override the default output from pydantic by calling `to_dict()` of filters
        if self.filters:
            _dict['filters'] = self.filters.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TraceAttributesRequest:
        """Create an instance of TraceAttributesRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TraceAttributesRequest.parse_obj(obj)

        _obj = TraceAttributesRequest.parse_obj({
            "id": obj.get("id"),
            "shape": [MapMatchWaypoint.from_dict(_item) for _item in obj.get("shape")] if obj.get("shape") is not None else None,
            "encoded_polyline": obj.get("encoded_polyline"),
            "costing": obj.get("costing"),
            "costing_options": CostingOptions.from_dict(obj.get("costing_options")) if obj.get("costing_options") is not None else None,
            "shape_match": obj.get("shape_match"),
            "directions_options": DirectionsOptions.from_dict(obj.get("directions_options")) if obj.get("directions_options") is not None else None,
            "filters": TraceAttributesRequestAllOfFilters.from_dict(obj.get("filters")) if obj.get("filters") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List
from pydantic import BaseModel, Field, StrictInt, StrictStr, conlist
from stadiamaps.models.route_leg import RouteLeg
from stadiamaps.models.route_summary import RouteSummary
from stadiamaps.models.routing_response_waypoint import RoutingResponseWaypoint
from stadiamaps.models.valhalla_languages import ValhallaLanguages
from stadiamaps.models.valhalla_long_units import ValhallaLongUnits

class RouteResponseTrip(BaseModel):
    """
    RouteResponseTrip
    """
    status: StrictInt = Field(..., description="The response status code")
    status_message: StrictStr = Field(..., description="The response status message")
    units: ValhallaLongUnits = Field(...)
    language: ValhallaLanguages = Field(...)
    locations: conlist(RoutingResponseWaypoint) = Field(...)
    legs: conlist(RouteLeg) = Field(...)
    summary: RouteSummary = Field(...)
    additional_properties: Dict[str, Any] = {}
    __properties = ["status", "status_message", "units", "language", "locations", "legs", "summary"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RouteResponseTrip:
        """Create an instance of RouteResponseTrip from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in locations (list)
        _items = []
        if self.locations:
            for _item in self.locations:
                if _item:
                    _items.append(_item.to_dict())
            _dict['locations'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in legs (list)
        _items = []
        if self.legs:
            for _item in self.legs:
                if _item:
                    _items.append(_item.to_dict())
            _dict['legs'] = _items
        # override the default output from pydantic by calling `to_dict()` of summary
        if self.summary:
            _dict['summary'] = self.summary.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RouteResponseTrip:
        """Create an instance of RouteResponseTrip from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RouteResponseTrip.parse_obj(obj)

        _obj = RouteResponseTrip.parse_obj({
            "status": obj.get("status"),
            "status_message": obj.get("status_message"),
            "units": obj.get("units"),
            "language": obj.get("language"),
            "locations": [RoutingResponseWaypoint.from_dict(_item) for _item in obj.get("locations")] if obj.get("locations") is not None else None,
            "legs": [RouteLeg.from_dict(_item) for _item in obj.get("legs")] if obj.get("legs") is not None else None,
            "summary": RouteSummary.from_dict(obj.get("summary")) if obj.get("summary") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj




# coding: utf-8

# flake8: noqa
"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import models into model package
from stadiamaps.models.access import Access
from stadiamaps.models.admin_region import AdminRegion
from stadiamaps.models.administrative import Administrative
from stadiamaps.models.auto_costing_options import AutoCostingOptions
from stadiamaps.models.auto_costing_options_all_of import AutoCostingOptionsAllOf
from stadiamaps.models.base_costing_options import BaseCostingOptions
from stadiamaps.models.base_trace_request import BaseTraceRequest
from stadiamaps.models.bicycle_costing_options import BicycleCostingOptions
from stadiamaps.models.bicycle_costing_options_all_of import BicycleCostingOptionsAllOf
from stadiamaps.models.bike_network import BikeNetwork
from stadiamaps.models.contour import Contour
from stadiamaps.models.coordinate import Coordinate
from stadiamaps.models.costing_model import CostingModel
from stadiamaps.models.costing_options import CostingOptions
from stadiamaps.models.directions_options import DirectionsOptions
from stadiamaps.models.distance_unit import DistanceUnit
from stadiamaps.models.edge_sign import EdgeSign
from stadiamaps.models.edge_use import EdgeUse
from stadiamaps.models.end_node import EndNode
from stadiamaps.models.geo_attributes import GeoAttributes
from stadiamaps.models.geo_json_geometry import GeoJSONGeometry
from stadiamaps.models.geo_json_geometry_base import GeoJSONGeometryBase
from stadiamaps.models.geo_json_line_string import GeoJSONLineString
from stadiamaps.models.geo_json_line_string_all_of import GeoJSONLineStringAllOf
from stadiamaps.models.geo_json_point import GeoJSONPoint
from stadiamaps.models.geo_json_point_all_of import GeoJSONPointAllOf
from stadiamaps.models.geo_json_polygon import GeoJSONPolygon
from stadiamaps.models.geo_json_polygon_all_of import GeoJSONPolygonAllOf
from stadiamaps.models.geocoding_object import GeocodingObject
from stadiamaps.models.height_request import HeightRequest
from stadiamaps.models.height_response import HeightResponse
from stadiamaps.models.highway_classification import HighwayClassification
from stadiamaps.models.intersecting_edge import IntersectingEdge
from stadiamaps.models.isochrone_costing_model import IsochroneCostingModel
from stadiamaps.models.isochrone_feature import IsochroneFeature
from stadiamaps.models.isochrone_properties import IsochroneProperties
from stadiamaps.models.isochrone_request import IsochroneRequest
from stadiamaps.models.isochrone_response import IsochroneResponse
from stadiamaps.models.locate_detailed_edge import LocateDetailedEdge
from stadiamaps.models.locate_edge import LocateEdge
from stadiamaps.models.locate_edge_info import LocateEdgeInfo
from stadiamaps.models.locate_node import LocateNode
from stadiamaps.models.locate_node_all_of import LocateNodeAllOf
from stadiamaps.models.locate_object import LocateObject
from stadiamaps.models.maneuver_sign import ManeuverSign
from stadiamaps.models.maneuver_sign_element import ManeuverSignElement
from stadiamaps.models.map_match_costing_model import MapMatchCostingModel
from stadiamaps.models.map_match_request import MapMatchRequest
from stadiamaps.models.map_match_request_all_of import MapMatchRequestAllOf
from stadiamaps.models.map_match_route_response import MapMatchRouteResponse
from stadiamaps.models.map_match_route_response_all_of import MapMatchRouteResponseAllOf
from stadiamaps.models.map_match_trace_options import MapMatchTraceOptions
from stadiamaps.models.map_match_waypoint import MapMatchWaypoint
from stadiamaps.models.map_match_waypoint_all_of import MapMatchWaypointAllOf
from stadiamaps.models.matched_point import MatchedPoint
from stadiamaps.models.matrix_costing_model import MatrixCostingModel
from stadiamaps.models.matrix_distance import MatrixDistance
from stadiamaps.models.matrix_request import MatrixRequest
from stadiamaps.models.matrix_response import MatrixResponse
from stadiamaps.models.motor_scooter_costing_options import MotorScooterCostingOptions
from stadiamaps.models.motor_scooter_costing_options_all_of import MotorScooterCostingOptionsAllOf
from stadiamaps.models.motorcycle_costing_options import MotorcycleCostingOptions
from stadiamaps.models.motorcycle_costing_options_all_of import MotorcycleCostingOptionsAllOf
from stadiamaps.models.nearest_roads_request import NearestRoadsRequest
from stadiamaps.models.node_id import NodeId
from stadiamaps.models.node_type import NodeType
from stadiamaps.models.optimized_route_request import OptimizedRouteRequest
from stadiamaps.models.pedestrian_costing_options import PedestrianCostingOptions
from stadiamaps.models.pelias_geo_json_feature import PeliasGeoJSONFeature
from stadiamaps.models.pelias_geo_json_properties import PeliasGeoJSONProperties
from stadiamaps.models.pelias_geo_json_properties_addendum import PeliasGeoJSONPropertiesAddendum
from stadiamaps.models.pelias_geo_json_properties_addendum_osm import PeliasGeoJSONPropertiesAddendumOsm
from stadiamaps.models.pelias_layer import PeliasLayer
from stadiamaps.models.pelias_response import PeliasResponse
from stadiamaps.models.pelias_response_geocoding import PeliasResponseGeocoding
from stadiamaps.models.pelias_source import PeliasSource
from stadiamaps.models.restrictions import Restrictions
from stadiamaps.models.road_class import RoadClass
from stadiamaps.models.route_leg import RouteLeg
from stadiamaps.models.route_maneuver import RouteManeuver
from stadiamaps.models.route_request import RouteRequest
from stadiamaps.models.route_response import RouteResponse
from stadiamaps.models.route_response_trip import RouteResponseTrip
from stadiamaps.models.route_summary import RouteSummary
from stadiamaps.models.routing_response_waypoint import RoutingResponseWaypoint
from stadiamaps.models.routing_response_waypoint_all_of import RoutingResponseWaypointAllOf
from stadiamaps.models.routing_waypoint import RoutingWaypoint
from stadiamaps.models.routing_waypoint_all_of import RoutingWaypointAllOf
from stadiamaps.models.routing_waypoint_all_of_search_filter import RoutingWaypointAllOfSearchFilter
from stadiamaps.models.simple_routing_waypoint import SimpleRoutingWaypoint
from stadiamaps.models.simple_routing_waypoint_all_of import SimpleRoutingWaypointAllOf
from stadiamaps.models.speeds import Speeds
from stadiamaps.models.trace_attribute_filter_options import TraceAttributeFilterOptions
from stadiamaps.models.trace_attribute_key import TraceAttributeKey
from stadiamaps.models.trace_attributes_base_response import TraceAttributesBaseResponse
from stadiamaps.models.trace_attributes_request import TraceAttributesRequest
from stadiamaps.models.trace_attributes_request_all_of import TraceAttributesRequestAllOf
from stadiamaps.models.trace_attributes_request_all_of_filters import TraceAttributesRequestAllOfFilters
from stadiamaps.models.trace_attributes_response import TraceAttributesResponse
from stadiamaps.models.trace_attributes_response_all_of import TraceAttributesResponseAllOf
from stadiamaps.models.trace_edge import TraceEdge
from stadiamaps.models.travel_mode import TravelMode
from stadiamaps.models.traversability import Traversability
from stadiamaps.models.truck_costing_options import TruckCostingOptions
from stadiamaps.models.truck_costing_options_all_of import TruckCostingOptionsAllOf
from stadiamaps.models.tz_response import TzResponse
from stadiamaps.models.valhalla_languages import ValhallaLanguages
from stadiamaps.models.valhalla_long_units import ValhallaLongUnits
from stadiamaps.models.warning import Warning



# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.  # noqa: E501

    The version of the OpenAPI document: 5.0.4
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class RoadClass(str, Enum):
    """
    Class of road (ranked in descending order)
    """

    """
    allowed enum values
    """
    MOTORWAY = 'motorway'
    TRUNK = 'trunk'
    PRIMARY = 'primary'
    SECONDARY = 'secondary'
    TERTIARY = 'tertiary'
    UNCLASSIFIED = 'unclassified'
    RESIDENTIAL = 'residential'
    SERVICE_OTHER = 'service_other'

    @classmethod
    def from_json(cls, json_str: str) -> RoadClass:
        """Create an instance of RoadClass from a JSON string"""
        return RoadClass(json.loads(json_str))


